@@ -46,22 +46,12 @@ class MenuEditor:
 			self.applications.dom = xml.dom.minidom.parse(self.applications.path)
 		self.__remove_whilespace_nodes(self.applications.dom)
 
-		self.settings = Menu()
-		self.settings.tree = gmenu.lookup_tree('settings.menu', gmenu.FLAGS_SHOW_EMPTY|gmenu.FLAGS_INCLUDE_EXCLUDED|gmenu.FLAGS_INCLUDE_NODISPLAY)
-		self.settings.visible_tree = gmenu.lookup_tree('settings.menu')
-		self.settings.path = os.path.join(util.getUserMenuPath(), self.settings.tree.get_menu_file())
-		if not os.path.isfile(self.settings.path):
-			self.settings.dom = xml.dom.minidom.parseString(util.getUserMenuXml(self.settings.tree))
-		else:
-			self.settings.dom = xml.dom.minidom.parse(self.settings.path)
-		self.__remove_whilespace_nodes(self.settings.dom)
 		self.save(True)
 
 	def save(self, from_loading=False):
-		for menu in ('applications', 'settings'):
-			fd = open(getattr(self, menu).path, 'w')
-			fd.write(re.sub("\n[\s]*([^\n<]*)\n[\s]*</", "\\1</", getattr(self, menu).dom.toprettyxml().replace('<?xml version="1.0" ?>\n', '')))
-			fd.close()
+		fd = open(self.applications.path, 'w')
+		fd.write(re.sub("\n[\s]*([^\n<]*)\n[\s]*</", "\\1</", self.applications.dom.toprettyxml().replace('<?xml version="1.0" ?>\n', '')))
+		fd.close()
 		if not from_loading:
 			self.__loadMenus()
 
@@ -80,22 +70,21 @@ class MenuEditor:
 				os.unlink(file_path)
 
 	def revert(self):
-		for name in ('applications', 'settings'):
-			menu = getattr(self, name)
-			self.revertTree(menu.tree.root)
-			path = os.path.join(util.getUserMenuPath(), menu.tree.get_menu_file())
-			try:
-				os.unlink(path)
-			except OSError:
-				pass
-			#reload DOM for each menu
-			if not os.path.isfile(menu.path):
-				menu.dom = xml.dom.minidom.parseString(util.getUserMenuXml(menu.tree))
-			else:
-				menu.dom = xml.dom.minidom.parse(menu.path)
-			self.__remove_whilespace_nodes(menu.dom)
-			#reset undo/redo, no way to recover from this
-			self.__undo, self.__redo = [], []
+		menu = self.applications
+		self.revertTree(menu.tree.root)
+		path = os.path.join(util.getUserMenuPath(), menu.tree.get_menu_file())
+		try:
+			os.unlink(path)
+		except OSError:
+			pass
+		#reload DOM for each menu
+		if not os.path.isfile(menu.path):
+			menu.dom = xml.dom.minidom.parseString(util.getUserMenuXml(menu.tree))
+		else:
+			menu.dom = xml.dom.minidom.parse(menu.path)
+		self.__remove_whilespace_nodes(menu.dom)
+		#reset undo/redo, no way to recover from this
+		self.__undo, self.__redo = [], []
 		self.save()
 
 	def revertTree(self, menu):
@@ -121,13 +110,12 @@ class MenuEditor:
 			os.unlink(file_path)
 			redo.append(redo_path)
 		#reload DOM to make changes stick
-		for name in ('applications', 'settings'):
-			menu = getattr(self, name)
-			if not os.path.isfile(menu.path):
-				menu.dom = xml.dom.minidom.parseString(util.getUserMenuXml(menu.tree))
-			else:
-				menu.dom = xml.dom.minidom.parse(menu.path)
-			self.__remove_whilespace_nodes(menu.dom)
+		menu = self.applications
+		if not os.path.isfile(menu.path):
+			menu.dom = xml.dom.minidom.parseString(util.getUserMenuXml(menu.tree))
+		else:
+			menu.dom = xml.dom.minidom.parse(menu.path)
+		self.__remove_whilespace_nodes(menu.dom)
 		self.__redo.append(redo)
 
 	def redo(self):
@@ -145,19 +133,17 @@ class MenuEditor:
 			os.unlink(file_path)
 			undo.append(undo_path)
 		#reload DOM to make changes stick
-		for name in ('applications', 'settings'):
-			menu = getattr(self, name)
-			if not os.path.isfile(menu.path):
-				menu.dom = xml.dom.minidom.parseString(util.getUserMenuXml(menu.tree))
-			else:
-				menu.dom = xml.dom.minidom.parse(menu.path)
-			self.__remove_whilespace_nodes(menu.dom)
+		menu = self.applications
+		if not os.path.isfile(menu.path):
+			menu.dom = xml.dom.minidom.parseString(util.getUserMenuXml(menu.tree))
+		else:
+			menu.dom = xml.dom.minidom.parse(menu.path)
+		self.__remove_whilespace_nodes(menu.dom)
 		self.__undo.append(undo)
 
 	def getMenus(self, parent=None):
 		if parent == None:
 			yield self.applications.tree.root
-			yield self.settings.tree.root
 		else:
 			for menu in parent.get_contents():
 				if menu.get_type() == gmenu.TYPE_DIRECTORY:
@@ -217,6 +203,10 @@ class MenuEditor:
 
 	def createItem(self, parent, icon, name, comment, command, use_term, before=None, after=None):
 		file_id = self.__writeItem(None, icon, name, comment, command, use_term)
+		self.insertExternalItem(file_id, parent.menu_id, before, after)
+
+	def insertExternalItem(self, file_id, parent_id, before=None, after=None):
+		parent = self.__findMenu(parent_id)
 		dom = self.__getMenu(parent).dom
 		self.__addItem(parent, file_id, dom)
 		self.__positionItem(parent, ('Item', file_id), before, after)
@@ -225,7 +215,11 @@ class MenuEditor:
 
 	def createMenu(self, parent, icon, name, comment, before=None, after=None):
 		file_id = self.__writeMenu(None, icon, name, comment)
+		self.insertExternalMenu(file_id, parent.menu_id, before, after)
+
+	def insertExternalMenu(self, file_id, parent_id, before=None, after=None):
 		menu_id = file_id.rsplit('.', 1)[0]
+		parent = self.__findMenu(parent_id)
 		dom = self.__getMenu(parent).dom
 		self.__addXmlDefaultLayout(self.__getXmlMenu(self.__getPath(parent), dom, dom) , dom)
 		menu_xml = self.__getXmlMenu(self.__getPath(parent) + '/' + menu_id, dom, dom)
@@ -407,39 +401,29 @@ class MenuEditor:
 			self.__undo[-1].append(undo_path)
 
 	def __getMenu(self, item):
-		root = item.get_parent()
-		if not root:
-			#already at the top
-			root = item
-		else:
-			while True:
-				if root.get_parent():
-					root = root.get_parent()
-				else:
-					break
-		if root.menu_id == self.applications.tree.root.menu_id:
-			return self.applications
-		return self.settings
+		return self.applications
+
+	def __findMenu(self, menu_id, parent=None):
+		if parent == None:
+			return self.__findMenu(menu_id, self.applications.tree.root)
+		if menu_id == self.applications.tree.root.menu_id:
+			return self.applications.tree.root
+		for item in parent.get_contents():
+			if item.get_type() == gmenu.TYPE_DIRECTORY:
+				if item.menu_id == menu_id:
+					return item
+				menu = self.__findMenu(menu_id, item)
+				if menu != None:
+					return menu
 
 	def __isVisible(self, item):
 		if item.get_type() == gmenu.TYPE_ENTRY:
 			return not (item.get_is_excluded() or item.get_is_nodisplay())
-		def loop_for_menu(parent, menu):
-			for item in parent.get_contents():
-				if item.get_type() == gmenu.TYPE_DIRECTORY:
-					if item.menu_id == menu.menu_id:
-						return True
-					temp = loop_for_menu(item, menu)
-					if temp:
-						return True
-			return False
 		menu = self.__getMenu(item)
-		if menu == self.applications:
-			root = self.applications.visible_tree.root
-		elif menu == self.settings:
-			root = self.settings.visible_tree.root
+		root = self.applications.visible_tree.root
 		if item.get_type() == gmenu.TYPE_DIRECTORY:
-			return loop_for_menu(root, item)
+			if self.__findMenu(item.menu_id, root) == None:
+				return False
 		return True
 
 	def __getPath(self, menu, path=None):
@@ -514,7 +498,7 @@ class MenuEditor:
 			file_id = item.get_desktop_file_id()
 			keyfile = util.DesktopParser(file_path)
 		elif item == None and name == None:
-			raise Exception(_('New menu items need a name'))
+			raise Exception('New menu items need a name')
 		else:
 			file_id = util.getUniqueFileId(name, '.desktop')
 			keyfile = util.DesktopParser()
@@ -547,7 +531,7 @@ class MenuEditor:
 			file_path = menu.get_desktop_file_path()
 			keyfile = util.DesktopParser(file_path)
 		elif menu == None and name == None:
-			raise Exception(_('New menus need a name'))
+			raise Exception('New menus need a name')
 		else:
 			file_id = util.getUniqueFileId(name, '.directory')
 			keyfile = util.DesktopParser(file_type='Directory')
